@@ -23,7 +23,7 @@
 
 //Alternatively, windows users with VisualStudio can compile this project
 // vcvarsall amd64
-// cl /EHsc main_console.cpp nii_dicom.cpp jpg_0XC3.cpp ujpeg.cpp nifti1_io_core.cpp nii_ortho.cpp nii_dicom_batch.cpp -DmyDisableOpenJPEG -DmyDisableJasper /odcm2niix
+// cl /EHsc main_console.cpp nii_foreign.cpp nii_dicom.cpp jpg_0XC3.cpp ujpeg.cpp nifti1_io_core.cpp nii_ortho.cpp nii_dicom_batch.cpp -DmyDisableOpenJPEG /o dcm2niix
 
 
 //#define mydebugtest //automatically process directory specified in main, ignore input arguments
@@ -71,16 +71,23 @@ void showHelp(const char * argv[], struct TDCMopts opts) {
     const char *cstr = removePath(argv[0]);
     printf("usage: %s [options] <in_folder>\n", cstr);
     printf(" Options :\n");
-    printf("  -1..-9 : gz compression level (1=fastest, 9=smallest)\n");
-    printf("  -b : BIDS sidecar (y/n, default n)\n");
+    printf("  -1..-9 : gz compression level (1=fastest..9=smallest, default %d)\n", opts.gzLevel);
+    char bidsCh = 'n';
+    if (opts.isCreateBIDS) bidsCh = 'y';
+    printf("  -b : BIDS sidecar (y/n/o(o=only: no NIfTI), default %c)\n", bidsCh);
+    if (opts.isAnonymizeBIDS) bidsCh = 'y'; else bidsCh = 'n';
+    printf("   -ba : anonymize BIDS (y/n, default %c)\n", bidsCh);
+    printf("  -c : comment stored as NIfTI aux_file (up to 24 characters)\n");
+    if (opts.isSortDTIbyBVal) bidsCh = 'y'; else bidsCh = 'n';
+    printf("  -d : diffusion volumes sorted by b-value (y/n, default %c)\n", bidsCh);
     #ifdef mySegmentByAcq
-    printf("  -f : filename (%%a=antenna  (coil) number, %%c=comments, %%d=description, %%e echo number, %%f=folder name, %%i ID of patient, %%m=manufacturer, %%n=name of patient, %%p=protocol, %%q=sequence number, %%s=series number, %%t=time, %%u=acquisition number, %%z sequence name; default '%s')\n",opts.filename);
+     #define kQstr " %%q=sequence number,"
     #else
-    printf("  -f : filename (%%a=antenna  (coil) number, %%c=comments, %%d=description, %%e echo number, %%f=folder name, %%i ID of patient, %%m=manufacturer, %%n=name of patient, %%p=protocol, %%s=series number, %%t=time, %%u=acquisition number, %%z sequence name; default '%s')\n",opts.filename);
+     #define kQstr ""
     #endif
+    printf("  -f : filename (%%a=antenna  (coil) number, %%c=comments, %%d=description, %%e echo number, %%f=folder name, %%i ID of patient, %%j seriesInstanceUID, %%k studyInstanceUID, %%m=manufacturer, %%n=name of patient, %%p=protocol,%s %%s=series number, %%t=time, %%u=acquisition number, %%v=vendor, %%x=study ID; %%z sequence name; default '%s')\n", kQstr, opts.filename);
     printf("  -h : show help\n");
-    printf("  -i : ignore derived and 2D images (y/n, default n)\n");
-    printf("  -t : text notes includes private patient details (y/n, default n)\n");
+    printf("  -i : ignore derived, localizer and 2D images (y/n, default n)\n");
     printf("  -m : merge 2D slices from same series regardless of study time, echo, coil, orientation, etc. (y/n, default n)\n");
     printf("  -o : output directory (omit to save to input folder)\n");
     printf("  -p : Philips precise float (not display) scaling (y/n, default y)\n");
@@ -92,14 +99,14 @@ void showHelp(const char * argv[], struct TDCMopts opts) {
     if (opts.isGz) gzCh = 'y';
     #ifdef myDisableZLib
 		if (strlen(opts.pigzname) > 0)
-			printf("  -z : gz compress images (y/n, default %c)\n", gzCh);
+			printf("  -z : gz compress images (y/n/3, default %c) [y=pigz, n=no, 3=no,3D]\n", gzCh);
 		else
-			printf("  -z : gz compress images (y/n, default %c) [REQUIRES pigz]\n", gzCh);
+			printf("  -z : gz compress images (y/n/3, default %c)  [y=pigz(MISSING!), n=no, 3=no,3D]\n", gzCh);
     #else
     	#ifdef myDisableMiniZ
-    	printf("  -z : gz compress images (y/i/n, default %c) [y=pigz, i=internal:zlib, n=no]\n", gzCh);
+    	printf("  -z : gz compress images (y/i/n/3, default %c) [y=pigz, i=internal:zlib, n=no, 3=no,3D]\n", gzCh);
 		#else
-		printf("  -z : gz compress images (y/i/n, default %c) [y=pigz, i=internal, n=no]\n", gzCh);
+		printf("  -z : gz compress images (y/i/n/3, default %c) [y=pigz, i=internal, n=no, 3=no,3D]\n", gzCh);
 		#endif
     #endif
 
@@ -107,6 +114,7 @@ void showHelp(const char * argv[], struct TDCMopts opts) {
     printf(" Defaults stored in Windows registry\n");
     printf(" Examples :\n");
     printf("  %s c:\\DICOM\\dir\n", cstr);
+    printf("  %s -c \"my comment\" c:\\DICOM\\dir\n", cstr);
     printf("  %s -o c:\\out\\dir c:\\DICOM\\dir\n", cstr);
     printf("  %s -f mystudy%%s c:\\DICOM\\dir\n", cstr);
     printf("  %s -o \"c:\\dir with spaces\\dir\" c:\\dicomdir\n", cstr);
@@ -114,14 +122,30 @@ void showHelp(const char * argv[], struct TDCMopts opts) {
     printf(" Defaults file : %s\n", opts.optsname);
     printf(" Examples :\n");
     printf("  %s /Users/chris/dir\n", cstr);
+    printf("  %s -c \"my comment\" /Users/chris/dir\n", cstr);
     printf("  %s -o /users/cr/outdir/ -z y ~/dicomdir\n", cstr);
+    printf("  %s -f %%p_%%s -b y -ba n ~/dicomdir\n", cstr);
     printf("  %s -f mystudy%%s ~/dicomdir\n", cstr);
     printf("  %s -o \"~/dir with spaces/dir\" ~/dicomdir\n", cstr);
 #endif
 } //showHelp()
 
-//#define mydebugtest
+int invalidParam(int i, const char * argv[]) {
+	if ((argv[i][0] == 'y') || (argv[i][0] == 'Y')
+		|| (argv[i][0] == 'n') || (argv[i][0] == 'N')
+		|| (argv[i][0] == 'o') || (argv[i][0] == 'O')
+		|| (argv[i][0] == 'h') || (argv[i][0] == 'H')
+		|| (argv[i][0] == 'i') || (argv[i][0] == 'I')
+		|| (argv[i][0] == '0') || (argv[i][0] == '1')
+		|| (argv[i][0] == '2') || (argv[i][0] == '3') )
+		return 0;
 
+	//if (argv[i][0] != '-') return 0;
+	printf(" Error: invalid option '%s %s'\n", argv[i-1], argv[i]);
+	return 1;
+}
+
+//#define mydebugtest
 int main(int argc, const char * argv[])
 {
     struct TDCMopts opts;
@@ -143,7 +167,7 @@ int main(int argc, const char * argv[])
         showHelp(argv, opts);
         return 0;
     }
-    //bool isCustomOutDir = false;
+    //for (int i = 1; i < argc; i++) { printf(" argument %d= '%s'\n", i, argv[i]);}
     int i = 1;
     int lastCommandArg = 0;
     while (i < (argc)) { //-1 as final parameter is DICOM directory
@@ -155,25 +179,52 @@ int main(int argc, const char * argv[])
             	if (opts.gzLevel > 11)
         	 		opts.gzLevel = 11;
             } else if ((argv[i][1] == 'b') && ((i+1) < argc)) {
+                if (strlen(argv[i]) < 3) { //"-b y"
+                	i++;
+                	if (invalidParam(i, argv)) return 0;
+                	if ((argv[i][0] == 'n') || (argv[i][0] == 'N')  || (argv[i][0] == '0'))
+                    	opts.isCreateBIDS = false;
+                	else {
+                    	opts.isCreateBIDS = true;
+                    	if ((argv[i][0] == 'o') || (argv[i][0] == 'O'))
+                    		opts.isOnlyBIDS = true;
+                    }
+                } else if (argv[i][2] == 'a') {//"-ba y"
+                	i++;
+                	if (invalidParam(i, argv)) return 0;
+                	if ((argv[i][0] == 'n') || (argv[i][0] == 'N')  || (argv[i][0] == '0'))
+                    	opts.isAnonymizeBIDS = false;
+                	else
+                    	opts.isAnonymizeBIDS = true;
+                } else
+                	printf("Error: Unknown command line argument: '%s'\n", argv[i]);
+            } else if ((argv[i][1] == 'c') && ((i+1) < argc)) {
                 i++;
+                snprintf(opts.imageComments,24,"%s",argv[i]);
+            } else if ((argv[i][1] == 'd') && ((i+1) < argc)) {
+                i++;
+                if (invalidParam(i, argv)) return 0;
                 if ((argv[i][0] == 'n') || (argv[i][0] == 'N')  || (argv[i][0] == '0'))
-                    opts.isCreateBIDS = false;
+                    opts.isSortDTIbyBVal = false;
                 else
-                    opts.isCreateBIDS = true;
+                    opts.isSortDTIbyBVal = true;
             } else if ((argv[i][1] == 'i') && ((i+1) < argc)) {
                 i++;
+                if (invalidParam(i, argv)) return 0;
                 if ((argv[i][0] == 'n') || (argv[i][0] == 'N')  || (argv[i][0] == '0'))
                     opts.isIgnoreDerivedAnd2D = false;
                 else
                     opts.isIgnoreDerivedAnd2D = true;
             } else if ((argv[i][1] == 'm') && ((i+1) < argc)) {
                 i++;
+                if (invalidParam(i, argv)) return 0;
                 if ((argv[i][0] == 'n') || (argv[i][0] == 'N')  || (argv[i][0] == '0'))
                     opts.isForceStackSameSeries = false;
                 else
                     opts.isForceStackSameSeries = true;
             } else if ((argv[i][1] == 'p') && ((i+1) < argc)) {
                 i++;
+                if (invalidParam(i, argv)) return 0;
                 if ((argv[i][0] == 'n') || (argv[i][0] == 'N')  || (argv[i][0] == '0'))
                     opts.isPhilipsFloatNotDisplayScaling = false;
                 else
@@ -181,18 +232,21 @@ int main(int argc, const char * argv[])
 
             } else if ((argv[i][1] == 's') && ((i+1) < argc)) {
                 i++;
+                if (invalidParam(i, argv)) return 0;
                 if ((argv[i][0] == 'n') || (argv[i][0] == 'N')  || (argv[i][0] == '0'))
                     opts.isOnlySingleFile = false;
                 else
                     opts.isOnlySingleFile = true;
             } else if ((argv[i][1] == 't') && ((i+1) < argc)) {
                 i++;
+                if (invalidParam(i, argv)) return 0;
                 if ((argv[i][0] == 'n') || (argv[i][0] == 'N')  || (argv[i][0] == '0'))
                     opts.isCreateText = false;
                 else
                     opts.isCreateText = true;
             } else if ((argv[i][1] == 'v') && ((i+1) < argc)) {
                 i++;
+                if (invalidParam(i, argv)) return 0;
                 if ((argv[i][0] == 'n') || (argv[i][0] == 'N')  || (argv[i][0] == '0')) //0: verbose OFF
                     opts.isVerbose = 0;
                 else if ((argv[i][0] == 'h') || (argv[i][0] == 'H')  || (argv[i][0] == '2')) //2: verbose HYPER
@@ -201,13 +255,18 @@ int main(int argc, const char * argv[])
                     opts.isVerbose = 1; //1: verbose ON
             } else if ((argv[i][1] == 'x') && ((i+1) < argc)) {
                 i++;
+                if (invalidParam(i, argv)) return 0;
                 if ((argv[i][0] == 'n') || (argv[i][0] == 'N')  || (argv[i][0] == '0'))
                     opts.isCrop = false;
                 else
                     opts.isCrop = true;
             } else if ((argv[i][1] == 'z') && ((i+1) < argc)) {
                 i++;
-                if ((argv[i][0] == 'i') || (argv[i][0] == 'I') ) {
+                if (invalidParam(i, argv)) return 0;
+                if ((argv[i][0] == '3') ) {
+                    opts.isGz = false; //uncompressed 3D
+                	opts.isSave3D = true;
+                } else if ((argv[i][0] == 'i') || (argv[i][0] == 'I') ) {
                     opts.isGz = true; //force use of internal compression instead of pigz
                 	strcpy(opts.pigzname,"");
                 } else if ((argv[i][0] == 'n') || (argv[i][0] == 'N')  || (argv[i][0] == '0'))
@@ -219,9 +278,9 @@ int main(int argc, const char * argv[])
                 strcpy(opts.filename,argv[i]);
             } else if ((argv[i][1] == 'o') && ((i+1) < argc)) {
                 i++;
-                //isCustomOutDir = true;
                 strcpy(opts.outdir,argv[i]);
-            }
+            } else
+             printf(" Error: invalid option '%s %s'\n", argv[i], argv[i+1]);;
             lastCommandArg = i;
         } //if parameter is a command
         i ++; //read next parameter
@@ -236,15 +295,21 @@ int main(int argc, const char * argv[])
         return EXIT_SUCCESS;
     }
 #endif
+	#ifndef myEnableMultipleInputs
+	if ((argc-lastCommandArg-1) > 1) {
+		printf("Warning: only processing last of %d input files (recompile with 'myEnableMultipleInputs' to recursively process multiple files)\n", argc-lastCommandArg-1);
+		lastCommandArg = argc - 2;
+	}
+	#endif
 	#if !defined(_WIN64) && !defined(_WIN32)
 	double startWall = get_wall_time();
 	#endif
     clock_t start = clock();
     for (i = (lastCommandArg+1); i < argc; i++) {
     	strcpy(opts.indir,argv[i]); // [argc-1]
-    	//if (!isCustomOutDir) strcpy(opts.outdir,opts.indir);
-    	if (nii_loadDir(&opts) != EXIT_SUCCESS)
-    		return EXIT_FAILURE;
+    	int ret = nii_loadDir(&opts);
+    	if (ret != EXIT_SUCCESS)
+    		return ret;
     }
     #if !defined(_WIN64) && !defined(_WIN32)
 		printf ("Conversion required %f seconds (%f for core code).\n",get_wall_time() - startWall, ((float)(clock()-start))/CLOCKS_PER_SEC);
